/*****************************************************************************/
/*
                                  SSI.h
*/
/*****************************************************************************/

#ifndef SSI_H_LOADED
#define SSI_H_LOADED 1

#include "wasd.h"

/* essentially depth of if-else-endif nesting */
#define SSI_MAX_FLOW_CONTROL 8

/* default sizeof read buffer for SSI files */
#define SSI_DEFAULT_USZ 4096

/**************/
/* structures */
/**************/

struct SsiVarStruct
{
   struct  ListEntryStruct  SsiVarList;
   int  NameLength,
        Size,
        ValueLength;
   char  *NamePtr,
         *ValuePtr;
   char  Data[1];
};

struct SsiTaskStruct
{
   struct ListEntryStruct  SsiTaskList;

   struct SsiTaskStruct  *RootDocumentTaskPtr;

   boolean  /* access to be expressed at "1st", "2nd", etc. */
            AccessOrdinal,
            /* SYSUAF authenticated user has been checked to have access */
            AuthVmsUserHasAccess,
            /* indexed by 'FlowControlIndex', flow has executed */
            FlowControlHasExecuted [SSI_MAX_FLOW_CONTROL],
            /* indexed by 'FlowControlIndex', flow is executing */
            FlowControlIsExecuting [SSI_MAX_FLOW_CONTROL],
            /* currently inside an "<--#ssi" statement */
            InsideSsiStatement,
            /* output occured in flow control (for carriage-control) */
            PageHasBeenOutput,
            /* an SSI error message was generated */
            SsiProblemEncountered,
            /* suppress blank lines result of flow control directive lines */
            SuppressLine,
            /* a variable has been output (for carriage-control) */
            TraceHasBeenOutput,
            /* trace on or off */
            TraceState,
            /* for SYSUAF-authenticated user */
            VmsUserHasAccess;

   int  /* number of time file accessed */
        AccessCount,
        /* status returned from sys$create() */
        AccessCreateStatus,
        /* */
        FileDetailsItem,
        /* */
        FileNameLength,
        /* line number currently being parsed (for error information) */
        LineNumber,
        /* index into flow-control nesting arrays */
        FlowControlIndex,
        /* buffer for logical value */
        NetWriteBufferedEscapeHtml,
        /* line number where current directive began */
        StatementLineNumber,
        /* length of SSI statement ;^) */
        StatementLength,
        /* buffer for most recent task status */
        TaskStatusBuffer;
        
   int  /* indexed by 'FlowControlIndex', series of flow control states */
        FlowControlState [SSI_MAX_FLOW_CONTROL];

   char  /* */
         AccessSinceText [128],
         /* buffer for SSI statement */
         Statement [1024],
         /* */
         FileName [256],
         /* */
         ScratchExpandedFileName [256],
         /* */
         ScratchFileName [256],
         /* */
         ScratchFormat [64];

   char  /* points to the list of CGI variables */
         *CgiBufferPtr,
         /* message string when generating an SSI error */
         *ErrMsgPtr,
         /* current character position when parsing lines */
         *LineParsePtr,
         /* pointer to storage allocated for size format string */
         *SizeFmtPtr,
         /* current character position in SSI statement buffer */
         *StatementPtr,
         /* pointer to storage allocated for time format string */
         *TimeFmtPtr;
                                    
   /* structure for ACP info */
   struct FileAcpStruct  FileAcpData;

   struct FAB  FileFab;
   struct NAM  FileNam;
   struct RAB  FileRab;
   struct XABDAT  FileXabDat;
   struct XABFHC  FileXabFhc;
   struct XABPRO  FileXabPro;

   struct FAB  AcpFab;
   struct NAM  AcpNam;

   struct FAB  AccessFab;
   struct NAM  AccessNam;
   struct RAB  AccessRab;
   struct XABDAT  AccessXabDat;
   struct XABPRO  AccessXabPro;

   /* pointer to function, used for specifying the next task */
   void (*AccessAstFunctionPtr)(struct RequestStruct*);
   void (*NextTaskFunction)(struct RequestStruct*);
   void (*FileOpenErrorFunction)(struct RequestStruct*);

   /* user variable list */
   struct ListHeadStruct  SsiVarList;
};

/***********************/
/* function prototypes */
/***********************/

SsiAccessesConnectAst (struct RAB*);
SsiAccessesFileClosed (struct FAB*);
SsiAccessesGetAst (struct RAB*);
SsiAccessesParseAst (struct FAB*);
SsiAccessesUpdateAst (struct RAB*);
SsiBegin (struct RequestStruct*, void*, void*, char*);
SsiClosed (struct FAB*);
SsiConnectAst (struct RAB*);
SsiDoAccesses (struct RequestStruct*);
SsiDoDcl (struct RequestStruct*);
SsiDoDir (struct RequestStruct*);
SsiDoEcho (struct RequestStruct*);
SsiDoElif (struct RequestStruct*);
SsiDoElse (struct RequestStruct*);
SsiDoEnd (struct RequestStruct*);
SsiDoEndif (struct RequestStruct*);
SsiDoIf (struct RequestStruct*);
SsiDoInclude (struct RequestStruct*);
SsiDoPrintEnv (struct RequestStruct*);
SsiDoSet (struct RequestStruct*);
SsiDoTrace (struct RequestStruct*);
SsiFileDetails (struct RequestStruct*, int);
SsiFileDetailsParseAst (struct FAB*);
SsiFileDetailsAcpInfoAst (struct RequestStruct*);
SsiIncludeError (struct RequestStruct*);
SsiNextRecord (struct RequestStruct*);
SsiNextRecordAst (struct RAB*);
SsiOpenAst (struct FAB*);
SsiParse (struct RequestStruct*);
SsiParseLine (struct RequestStruct*);
SsiStatement (struct RequestStruct*);
SsiTraceGetVar (struct RequestStruct*, char*, char*);
SsiTraceLine (struct RequestStruct*);
SsiTraceSetVar (struct RequestStruct*, char*, char*);
char* SsiSysGetMsg (int);
char* SsiGetVar (struct RequestStruct*, char*, char*, boolean);
char* SsiGetCgiVar (struct RequestStruct*, char*);
char* SsiGetServerVar (struct RequestStruct*, char*, char*);
char* SsiGetUserVar (struct RequestStruct*, char*);
int SsiTimeString (struct RequestStruct*, unsigned long*, char*, char*, int);

#endif /* SSI_H_LOADED */

/*****************************************************************************/
